@@ -1,6 +1,21 @@
 'use strict';
 
-const WIDGET_SERIALIZER = '{ deserialize: widgets.unpack_models }';
+const JS_WIDGET_SERIALIZER = '{ deserialize: serializers.unpackThreeModel }';
+
+
+function pythonify(value) {
+    if (value === false) { return 'False'; }
+    if (value === true) { return 'True'; }
+    if (value === Infinity) { return "float('inf')"; }
+    if (value === -Infinity) { return "-float('inf')"; }
+    if (value === undefined || value === null) { return 'None'; }
+    if (Array.isArray(value)) {
+        return `[${
+            value.map(function(v) { return pythonify(v); }).join(', ')
+        }]`;
+    }
+    return JSON.stringify(value);
+}
 
 class BaseType {
     constructor(options) {
@@ -9,21 +24,19 @@ class BaseType {
         this.nullable = options.nullable === true;
     }
     getJSPropertyValue() {
+        if (this.defaultValue === Infinity || this.defaultValue === -Infinity) {
+            return this.defaultValue.toString();
+        }
+        if (this.defaultValue === undefined) {
+            return 'undefined';
+        }
         return JSON.stringify(this.defaultValue);
     }
     getPropArrayName() {
         return null;
     }
     getPythonDefaultValue() {
-        if (this.defaultValue === false) { return 'False'; }
-        if (this.defaultValue === true) { return 'True'; }
-        if (this.defaultValue === 0) { return '0'; }
-        if (this.defaultValue === '') { return "''"; }
-        if (this.defaultValue === Infinity) { return "float('inf')"; }
-        if (this.defaultValue === -Infinity) { return "-float('inf')"; }
-        if (!this.defaultValue) { return 'None'; }
-
-        return JSON.stringify(this.defaultValue);
+        return pythonify(this.defaultValue);
     }
     getPropertyConverterFn() {
         return null;
@@ -75,7 +88,7 @@ class ThreeType extends BaseType {
         super(options);
         this.typeName = typeName || '';
         this.defaultValue = null;
-        this.serializer = WIDGET_SERIALIZER;
+        this.serializer = JS_WIDGET_SERIALIZER;
         this.nullable = options.nullable !== false;
         this.args = options.args;
         this.kwargs = options.kwargs;
@@ -149,7 +162,7 @@ class ThreeTypeArray extends BaseType {
         super(options);
         this.typeName = typeName;
         this.defaultValue = [];
-        this.serializer = WIDGET_SERIALIZER;
+        this.serializer = JS_WIDGET_SERIALIZER;
         this.nullable = options.nullable !== false;
         this.allow_single = options.allow_single === true;
     }
@@ -185,7 +198,7 @@ class ThreeTypeDict extends BaseType {
         super(options);
         this.typeName = typeName;
         this.defaultValue = {};
-        this.serializer = WIDGET_SERIALIZER;
+        this.serializer = JS_WIDGET_SERIALIZER;
     }
     getTagParts() {
         return super.getTagParts().concat(['**widget_serialization']);
@@ -195,7 +208,8 @@ class ThreeTypeDict extends BaseType {
             const instances = this.typeName.map(function(typeName) {
                 return `        Instance(${typeName})`;
             });
-            return `Dict(Union([\n${instances.join(',\n')}\n    ]))${this.getTagString()}`;
+            return `Dict(Union([\n${
+                instances.join(',\n')}\n    ]))${this.getTagString()}`;
         }
         if (this.typeName === 'this') {
             return `Dict(This())${this.getTagString()}`;
@@ -214,7 +228,7 @@ class BufferMorphAttributes extends BaseType {
     constructor(options) {
         super(options);
         this.defaultValue = {};
-        this.serializer = WIDGET_SERIALIZER;
+        this.serializer = JS_WIDGET_SERIALIZER;
     }
     getTagParts() {
         return super.getTagParts().concat(['**widget_serialization']);
@@ -224,7 +238,8 @@ class BufferMorphAttributes extends BaseType {
         const instances = typeNames.map(function(typeName) {
             return `        Instance(${typeName})`;
         });
-        return 'Dict(Tuple(Union([\n' + instances.join(',\n') + '\n    ])))' + this.getTagString();
+        return 'Dict(TypedTuple(Union([\n' +
+            instances.join(',\n') + '\n    ])))' + this.getTagString();
     }
     getPropArrayName() {
         return 'three_nested_properties';
@@ -241,7 +256,8 @@ class Bool extends BaseType {
     }
     getTraitlet() {
         const nullableStr = this.getNullableStr();
-        return `Bool(${this.getPythonDefaultValue()}, ${nullableStr})${this.getTagString()}`;
+        return `Bool(${this.getPythonDefaultValue()}, ${nullableStr})${
+            this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertBool';
@@ -251,7 +267,7 @@ class Bool extends BaseType {
 class Int extends BaseType {
     constructor(defaultValue, options) {
         options = options || {};
-        super();
+        super(options);
         this.minValue = options.minValue;
         this.maxValue = options.maxValue;
         this.defaultValue = (defaultValue === null || defaultValue === undefined) && !this.nullable ? 0 : defaultValue ;
@@ -315,7 +331,9 @@ class Enum extends BaseType {
     }
     getTraitlet() {
         const nullableStr = this.getNullableStr();
-        return `Enum(${this.enumTypeName}, ${this.getPythonDefaultValue()}, ${nullableStr})${this.getTagString()}`;
+        return `Enum(${this.enumTypeName}, ${
+            this.getPythonDefaultValue()}, ${nullableStr})${
+            this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertEnum';
@@ -329,7 +347,9 @@ class Color extends BaseType {
     }
     getTraitlet() {
         const nullableStr = this.getNullableStr();
-        return `Unicode(${this.getPythonDefaultValue()}, ${nullableStr})${this.getTagString()}`;
+        return `Color(${
+            this.getPythonDefaultValue()}, ${nullableStr})${
+            this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertColor';
@@ -342,7 +362,8 @@ class ColorArray extends BaseType {
         this.defaultValue = defaultValue || ['#ffffff'];
     }
     getTraitlet() {
-        return `List(trait=Unicode(), default_value=${this.getPythonDefaultValue()})${this.getTagString()}`;
+        return `List(trait=Unicode(), default_value=${
+            this.getPythonDefaultValue()})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertColorArray';
@@ -371,16 +392,24 @@ class ArrayBufferType extends BaseType {
         super(options);
         this.arrayType = arrayType;
         this.shapeConstraint = shapeConstraint;
-        this.defaultValue = null;
+        this.defaultValue = options && options.nullable ? null : undefined;
         this.serializer = 'dataserializers.data_union_serialization';
     }
     getTraitlet() {
         const args = [];
+        if (this.defaultValue !== undefined) {
+            args.push(pythonify(this.defaultValue));
+        }
         if (this.arrayType) {
-            args.push(`dtype=${this.arrayType}`);
+            args.push(`dtype=${pythonify(this.arrayType)}`);
         }
         if (this.shapeConstraint) {
-            args.push(`shape_constraint=${this.shapeConstraint}`);
+            args.push(`shape_constraint=shape_constraints(${
+                this.shapeConstraint.map(function(v) { return pythonify(v); }).join(', ')
+            })`);
+        }
+        if (this.nullable) {
+            args.push(this.getNullableStr());
         }
 
         return `WebGLDataUnion(${args.join(', ')})${this.getTagString()}`;
@@ -400,7 +429,9 @@ class DictType extends BaseType {
     }
     getTraitlet() {
         const nullableStr = this.getNullableStr();
-        return `Dict(default_value=${this.getPythonDefaultValue()}, ${nullableStr})${this.getTagString()}`;
+        return `Dict(default_value=${
+            this.getPythonDefaultValue()}, ${nullableStr})${
+            this.getTagString()}`;
     }
     getPropertyAssignmentFn() {
         return 'assignDict';
@@ -408,8 +439,21 @@ class DictType extends BaseType {
 }
 
 class UniformDict extends DictType {
-    getPropertyConverterFn() {
-        return 'convertUniformDict';
+    constructor(defaultValue={}, options) {
+        super(defaultValue, options);
+        this.serializer = '{ serialize: serializers.serializeUniforms, ' +
+            'deserialize: serializers.deserializeUniforms }';
+    }
+    getTraitlet() {
+        const nullableStr = this.getNullableStr();
+        return `Dict(default_value=${
+            this.getPythonDefaultValue()
+        }, trait=Uniform(allow_none=True), ${
+            nullableStr
+        })${this.getTagString()}`;
+    }
+    getTagParts() {
+        return super.getTagParts().concat(['**uniforms_serialization']);
     }
 }
 
@@ -419,7 +463,8 @@ class FunctionType extends BaseType {
         this.defaultValue = fn || function() {};
     }
     getTraitlet() {
-        return `Unicode('${this.defaultValue.toString()}')${this.getTagString()}`;
+        return `Unicode('${
+            this.defaultValue.toString()}')${this.getTagString()}`;
     }
     getJSPropertyValue() {
         return this.defaultValue.toString();
@@ -435,7 +480,8 @@ class Vector2 extends BaseType {
         this.defaultValue = [ x||0, y||0 ];
     }
     getTraitlet() {
-        return `Vector2(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Vector2(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertVector';
@@ -451,7 +497,8 @@ class Vector3 extends BaseType {
         this.defaultValue = [ x||0, y||0, z||0 ];
     }
     getTraitlet() {
-        return `Vector3(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Vector3(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertVector';
@@ -467,7 +514,8 @@ class Vector4 extends BaseType {
         this.defaultValue = [ x||0, y||0, z||0, w||0 ];
     }
     getTraitlet() {
-        return `Vector4(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Vector4(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertVector';
@@ -499,7 +547,7 @@ class FaceArray extends BaseType {
         this.defaultValue = [];
     }
     getTraitlet() {
-        return `Tuple(trait=Face3())${this.getTagString()}`;
+        return `TypedTuple(trait=Face3())${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertFaceArray';
@@ -519,7 +567,8 @@ class Matrix3 extends BaseType {
         ];
     }
     getTraitlet() {
-        return `Matrix3(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Matrix3(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertMatrix';
@@ -540,7 +589,8 @@ class Matrix4 extends BaseType {
         ];
     }
     getTraitlet() {
-        return `Matrix4(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Matrix4(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertMatrix';
@@ -558,7 +608,8 @@ class Euler extends BaseType {
     }
 
     getTraitlet() {
-        return `Euler(default_value=${JSON.stringify(this.defaultValue)})${this.getTagString()}`;
+        return `Euler(default_value=${
+            pythonify(this.defaultValue)})${this.getTagString()}`;
     }
     getPropertyConverterFn() {
         return 'convertEuler';
